//////////////////////////////////////////////////////////////////////
// COMPUTER MOTION INCORPORATION                  COMPANY CONFIDENTIAL
//
// Class:   IOL_HP_CS (Hamid Protocol)
// Needs:   hp.cpp
//
// Version  When      Who               Why
// -------  ----      ---               ---
// 1.0      01/01/96  Daryl Fortney     Initial Design
//
// Copyright 1996 Computer Motion Incorporation.  All rights reserved.
//////////////////////////////////////////////////////////////////////
// Interface Declaration
//////////////////////////////////////////////////////////////////////
#ifndef HP_CS_H
#define HP_CS_H

// Dependencies
#include "Iol.hpp"
#include "Uart.hpp"

// Constants
#define	RESERVED	(unsigned char)0x00
#define	MAX_NUM		(unsigned char)250
#define	CS_KEY		(unsigned int)0xffa5
#define	HP_CS_MODE	(UART_38400B|UART_8W|UART_NP|UART_1S)
#define HP_TIMEOUT  100 // timeout in ms for receiving answer.
// Interface Declaration
class IOL_HP_CS{

	unsigned char	TX[MAX_NUM];
	long			TXL;
	unsigned int	TXCS;

	unsigned char	RX[MAX_NUM];
	long			RXL;
	unsigned int	RXCS;

	// Private Interface
	void	Pack(unsigned char d,unsigned char* p,long* num);
	long	Exception(unsigned char v);

	public:

	IOL_UART	L1;

	// Constructors
	IOL_HP_CS(void)					{ Reset(); }

	// Maintainance Functions
	void	Print();
	void	Reset(void)				{ TxReset(); RxReset(); }
	BOOL	Open(char *p,long m=HP_CS_MODE);
	BOOL	Close(void);

	// Tx Functions
	long	Tx(void* block,long num=1,long until=-1);
	long	TxPacket(void);
	void	TxReset(void)			{ TXL=TXCS=0; }

	// Rx Functions
	long	Rx(void* block,long num=1,long until=-1);
	long	RxPacket(void);
	void    RxReset(void)			{ RXL=RXCS=0; }

	// Old Protocol Compatibility
	int SerialInit(int buffers, int retries, long baud, int port);
	int SerialSend(char *pbuf, int num);
	int SerialReceive(char *pbuf);
};

#endif