//////////////////////////////////////////////////////////////////////
// UCSB - Electrical and Computer Engineering Dept.
//
// Input/Output Library Manager
//
// Version  When      Who               Why
// -------  ----      ---               ---
// 1.0      07/01/96  Daryl Fortney     Initial Design
//
// This builds the Input/Output Library.  The Input/Output Library
// tries to bring to terms all IO type operations and devices.  The
// IOL Hierarchy is shown below:
//
// iol---+---file (DOS & UNIX File Support)
//       +---hp_cs (Hamid Packet with Checksum)
//       +---hp_crc (Hamid Packet with CRC)
//       +---hnp (Hermes Network Protocol)
//       +---uart (DOS & UNIX Serial Port Support)
//       +---ptu (Directed Perceptions Pan Tilt Unit Support) > uart
//       +---slu (Servo Lens Unit Support) > uart
//       +---video (Video Card Support)
//       +---audio (Audio Card Support)
//
// Copyright 1996 UCSB.  All rights reserved.
//////////////////////////////////////////////////////////////////////
// Input/Output Library Manager
//////////////////////////////////////////////////////////////////////
#ifndef IOL_HPP
#define IOL_HPP

#define IOL_WINDOWS

#define	FALSE	0
#define	TRUE	1

// Generic Include Files
#include <ctype.h>
#include <iostream>
#include <iomanip>
#include <time.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <conio.h>
#include <math.h>
#include <process.h>

// IBM PC (Symantec Compiler)
#ifdef IOL_DOS_SC
	#define IOL_DOS
	#include <dos.h>
#endif

// IBM PC (Borland Compiler)
#ifdef IOL_DOS_BC
	#define IOL_DOS
	#include <dos.h>
#endif

// SGI UNIX (CC Compiler)
#ifdef IOL_SGI_CC
	#define IOL_UNIX
	#include <unistd.h>
	#include <fcntl.h>
	#include <termio.h>
#endif

// Windows 95 (Visual C++)
#ifdef IOL_WINDOWS
	using namespace std;
	#include <io.h>
	#include <windows.h>
	#include <winbase.h>
#endif

#define	BOOL	char
#define	BYTE	char
#define	WORD	long

// Event Timing
#ifndef	EVENT_TIMING

	#ifdef IOL_ALL
		#define IOL_CHECKS
		#define IOL_PRINTS
	#endif

	// Checking
	#ifdef	IOL_CHECKS
		#define IOL_FILE_CHECKS
		#define IOL_UART_CHECKS
		#define IOL_PTU_CHECKS
		#define IOL_SLU_CHECKS
		#define IOL_VIDEO_CHECKS
		#define IOL_AUDIO_CHECKS
	#endif

	// Printing
	#ifdef	IOL_PRINTS
		#define IOL_FILE_PRINTS
		#define IOL_UART_PRINTS
		#define IOL_PTU_PRINTS
		#define IOL_SLU_PRINTS
		#define IOL_VIDEO_PRINTS
		#define IOL_AUDIO_PRINTS
	#endif

	#ifdef	IOL_PRINT_FLOWS
		#define IOL_FILE_PRINT_FLOWS
		#define IOL_UART_PRINT_FLOWS
		#define IOL_PTU_PRINT_FLOWS
		#define IOL_SLU_PRINT_FLOWS
		#define IOL_VIDEO_PRINT_FLOWS
		#define IOL_AUDIO_PRINT_FLOWS
	#endif

	#ifdef	IOL_PRINT_ERRORS
		#define IOL_FILE_PRINT_ERRORS
		#define IOL_UART_PRINT_ERRORS
		#define IOL_PTU_PRINT_ERRORS
		#define IOL_SLU_PRINT_ERRORS
		#define IOL_VIDEO_PRINT_ERRORS
		#define IOL_AUDIO_PRINT_ERRORS
	#endif

#endif

// Error Handler Constants
#define	IOL_OPEN	0			// Device Open
#define	IOL_CLOSED	-1			// Device Closed
#define	IOL_NOERR	0			// No Error
#define	IOL_ERROR	-1			// Error

// Position Mode Constants
#define	IOL_POS_B	0x0000		// Position From Beginning
#define	IOL_POS_C	0x0001		// Position From Current
#define	IOL_POS_E	0x0002		// Position From End

// ASCII Code Constants
#define	IOL_ALL	-1
#define	IOL_NUL	0x00
#define	IOL_SOH	0x01
#define	IOL_STX	0x02
#define	IOL_ETX	0x03
#define	IOL_EOT	0x04
#define	IOL_ENQ	0x05
#define	IOL_ACK	0x06
#define	IOL_BEL	0x07
#define	IOL_BS 	0x08
#define	IOL_HT 	0x09
#define	IOL_LF 	0x0A
#define	IOL_VT 	0x0B
#define	IOL_FF 	0x0C
#define	IOL_CR 	0x0D
#define	IOL_SO 	0x0E
#define	IOL_SI	0x0F
#define	IOL_DLE	0x10
#define	IOL_DC1	0x11
#define	IOL_DC2	0x12
#define	IOL_DC3	0x13
#define	IOL_DC4	0x14
#define	IOL_NAK	0x15
#define	IOL_SYN	0x16
#define	IOL_ETB	0x17
#define	IOL_CAN	0x18
#define	IOL_EM 	0x19
#define	IOL_SUB	0x1A
#define	IOL_ESC	0x1B
#define	IOL_FS 	0x1C
#define	IOL_GS 	0x1D
#define	IOL_RS 	0x1E
#define	IOL_US 	0x1F
#define	IOL_SP 	0x20

// Useful RTS Extensions
#define	cerr cout
void DisplayGoto(int x, int y);
void DisplayClear2End(int x, int y);
void DisplayTextBox(int x, int y, int dx, int dy);
void Delay(long ms);
void Indent(long v);
void* Address(void*);
long StrSame(char* a,char* b);
long StrLength(char* a);

#endif
