/********************************************************
*														*
*  AESOP Development System								*
*														*
*  AESOP command routines (PC-side)						*
*														*
*  Authors: S. Jordan									*
*	 A. A. Mangaser (debug, completion, more features)	*
*	 H. Wasti (PC timeout routine)						*
*														*
*  Version: $Revision:   1.0  $							*
*														*
*  Copyright (C) 1994 Computer Motion Inc.				*
*														*
*********************************************************/

#include "a3000.h"

IOL_HP_CS	aesop;
BYTE InBuf[256];
BYTE OutBuf[256];
int length;

//////////////////////////////////////////////////////////////////////
// Initializes Robot Communications
//////////////////////////////////////////////////////////////////////
BOOL Initialize(long port){
	
	if(aesop.SerialInit(0,0,0,port)!=0)
		return FALSE;

	return TRUE;
}

//////////////////////////////////////////////////////////////////////
// Sends Data To Robot
//////////////////////////////////////////////////////////////////////
int Send(BYTE *pbuf, int num){
	return aesop.SerialSend(pbuf, num);
}


//////////////////////////////////////////////////////////////////////
// Receives Data From Robot
//////////////////////////////////////////////////////////////////////
int Receive(BYTE *pbuf){
	return aesop.SerialReceive(pbuf);
}

/*****************************************************************
* Function to pack floats into a BYTE array
******************************************************************/
void PacFloat(BYTE *OutBuf, int sub, float number) {
	int i;
	BYTE *ptr; /* Pointer to BYTE */
	ptr =(BYTE*) &number;
	for (i = 0; i < 4; i++) {
		OutBuf[sub + i] = *(ptr + i);
	}
}


/*****************************************************************
* Function to unpack floats from a BYTE array
******************************************************************/
void UnpacFloat(float *number, BYTE *InBuf, int sub){
	int i;
	BYTE *ptr;			/* Pointer to BYTE */
	ptr =(BYTE*)number;	/* Cast float pointer as BYTE pointer*/
	for (i = 0; i < 4; i++) {
		*(ptr + i) = InBuf[sub + i]; /* Byte copy */
	}
}




/*******************************************************************
*																   *
*   Function :  C4x_To_IEEE_32									   *
*																   *
*   Purpose  :  Convert a block of words from C4x float format	   *
*				to IEEE floating point format (in place).		   *
*																   *
*   Paramtrs :  2												   *
*				vp_memval   -									   *
*							address of start of memory containing  *
*							C4x float values					   *
*				u_length	-									   *
*							number of words to convert			   *
*																   *
*   Returns  :  NO_ERROR										   *
*																   *
*******************************************************************/
UINT C4x_To_IEEE_32(void *vp_mv, UINT u_length){
	union utag {
		float ieeerep;
		long  transrep;
	} pat;
	ULONG  c4xexp, c4xval, ieeeval;
	UINT   u_count;

	ULONG*	vp_memval=(ULONG*)vp_mv;

	for (u_count = 0; u_count < u_length; u_count++) {
		c4xval = *(vp_memval);
		if (c4xval == 0x80000000l) {
			ieeeval = 0;
		}
		else {
			c4xexp =((c4xval & 0xFF000000l) >> 24);
			if (c4xexp & 0x80) {
				c4xexp |= 0xFFFFFF00l;
			}
			c4xexp += 127;
			c4xexp &= 0xFFl;
			if (c4xval & 0x800000) {
				c4xval =(c4xval & 0x7FFFFF) | 0x1000000;
				c4xval = -c4xval;
				if ((c4xval & 0x800000) == 0) {
					c4xexp += 1;
				}
				ieeeval =(c4xexp << 23) +(c4xval & 0x7FFFFF) | 0x80000000;
			}
			else {
				c4xval = c4xval & 0x7FFFFF;
				c4xexp = c4xexp << 23;
				c4xval &= 0x7FFFFF;
				ieeeval = c4xexp + c4xval;
			}
		}
		pat.transrep = ieeeval;
		*((float *)vp_memval) = pat.ieeerep;
		vp_memval++;
	}
	return (NO_ERROR);
}


/*******************************************************************
*																   *
*   Function :  IEEE_To_C4x_32									   *
*																   *
*   Purpose  :  Convert a block of words from IEEE float format	   *
*				to C4x floating point format (in place).		   *
*																   *
*   Paramtrs :  2												   *
*				vp_memval   -									   *
*							address of start of memory containing  *
*							IEEE float values					   *
*				u_length	-									   *
*							number of words to convert			   *
*																   *
*   Returns  :  NO_ERROR										   *
*																   *
*******************************************************************/
UINT IEEE_To_C4x_32(void *vp_md, UINT u_length){
	union utag {
			float ieeerep;
			long  transrep;
	} pat;
	UINT   u_count;
	ULONG  c4xexp, c4xval, mantissa;
	
	ULONG*	vp_memdata=(ULONG*)vp_md;

	for (u_count = 0; u_count < u_length; u_count++) {
		pat.ieeerep = *((float *)vp_memdata);
		if (pat.transrep == 0) {
			c4xval = 0x80000000;
		}
		else {
			c4xexp = pat.transrep & 0x7F800000;
			c4xexp = c4xexp >> 23;
			c4xexp -= 127;
			if (pat.transrep & 0x80000000) {
				mantissa = pat.transrep & 0x7fffff;
				mantissa |= 0x800000;
				mantissa = -mantissa;
				if (mantissa & 0x800000)
					c4xexp += -1;
				mantissa |= 0x800000;
				c4xval = mantissa & 0xFFFFFF;
			}
			else {
				c4xval = pat.transrep & 0x7fffff;
			}
			c4xexp &= 0xFF;
			c4xexp = c4xexp << 24;
			c4xval = c4xval | c4xexp;
		}
		*(vp_memdata) = c4xval;
		vp_memdata++;
	}
	return (NO_ERROR);
}

extern unsigned CSum;
extern int Received;

/*****************************************************************
* Function for bouncing back floats to and from TMS.
******************************************************************/
int BounceFloats(float inf[6], float outf[6]){
	int i;
	int kb;
	BYTE *posPtr;
	int retval;

	OutBuf[0] = BOUNCE_FLOATS; /* Write command code	 */
	posPtr =(BYTE *)inf;
	for (i = 0; i < 6 * 4; i++) { /* Copy outgoing data	 */
		OutBuf[i + 1] = *(posPtr + i);
	}
	IEEE_To_C4x_32(&OutBuf[1], 6); /* Convert data	 */
	if ((retval = Send(OutBuf, 6 * 4 + 1)) != 0) { /* Send Command		 */
		if (retval == 2) 
			return (SEND_ERROR);
		else 
			return (SEND_LENGTH_ERROR);		   /* retval must be == 1 */
	}

	if ((length = Receive(InBuf)) == 0) 
		return (RECEIVE_ERROR);
	if (InBuf[0] != BOUNCE_FLOATS) { /* Check code		 */
		return (InBuf[0]);
	}
	if (length != 25) {
		return (LENGTH_ERROR); /* Check length	   */
	}
	posPtr =(BYTE *)outf;
	for (i = 0; i < 6 * 4; i++) { /* Copy incoming data	 */
		*(posPtr + i) = InBuf[i + 1];
	}
	C4x_To_IEEE_32(&outf[0], 6); /* Convert data	 */
	return (NO_ERROR);			 /* Return OK		 */
}


/*****************************************************************
* Function for returning joint position values
******************************************************************/
int JtPosReq(float position[NUMPOS]){
	int i;
	int kb;
	BYTE *posPtr =(BYTE *)position;
	int retval;

	OutBuf[0] = JT_POS_REQ;	/* Write command code	 */
	if ((retval = Send(OutBuf, 1)) != 0) { /* Send Command		 */
		if (retval == 2) 
			return (SEND_ERROR);
		else 
			return (SEND_LENGTH_ERROR);   /* retval must be == 1 */
	}

	if ((length = Receive(InBuf)) == 0) 
		return (RECEIVE_ERROR);
	if (InBuf[0] != JT_POS_REQ) { /* Check code		 */
		return (InBuf[0]);
	}
	if (length != 1+NUMPOS*4) {
		return (LENGTH_ERROR); /* Check length	   */
	}
	for (i = 0; i < NUMPOS * 4; i++) { /* Copy incoming data	 */
		*(posPtr + i) = InBuf[i + 1];
	}
	C4x_To_IEEE_32(&position[0], NUMPOS); /* Convert data	 */
	return (NO_ERROR);				 /* Return OK		 */
}


/*****************************************************************
* Function: CartTipPos
*	Gets cartesian tip position coordinates from AESOP
*	x,y,z (cartesian) t1,t2,t3 (Euler angles)
* Returns: 0 if OK. Error code if not
******************************************************************/
int CartTipPos(float position[NUMPOS]){
	int i;
	int kb;
	BYTE *posPtr =(BYTE *)position;
	int retval;

	OutBuf[0] = CART_TIP_POS; /* Write command code	 */
	if ((retval = Send(OutBuf, 1)) != 0) { /* Send Command		 */
		if (retval == 2) 
			return (SEND_ERROR);
		else 
			return (SEND_LENGTH_ERROR);   /* retval must be == 1 */
	}

	if ((length = Receive(InBuf)) == 0) 
		return (RECEIVE_ERROR);
	if (InBuf[0] != CART_TIP_POS) {
		return (InBuf[0]); /* Check code	   */
	}
	if (length != 1+NUMPOS*4) {
		return (LENGTH_ERROR); /* Check length	   */
	}
	for (i = 0; i < NUMPOS * 4; i++) { /* Copy incoming data	 */
		*(posPtr + i) = InBuf[i + 1];
	}
	C4x_To_IEEE_32(&position[0], NUMPOS); /* Convert data	 */
	return (NO_ERROR);				 /* Return OK		 */
}


/*****************************************************************
* Function: ContJointPos
*	Requests AESOP to continuously send packets of joint data.
* Returns: 0 if OK. Error code if not
******************************************************************/
int ContJointPos(float period){
	int retval;

	OutBuf[0] = CONT_JOINT_POS;	/* Write command code	 */
	if ((retval = Send(OutBuf, 1)) != 0) { /* Send Command		 */
		if (retval == 2) return (SEND_ERROR);
		else return (SEND_LENGTH_ERROR);   /* retval must be == 1 */
	}

	if ((length = Receive(InBuf)) == 0) return (RECEIVE_ERROR);
	if (InBuf[0] != CONT_JOINT_POS) return (InBuf[0]); /* Check code */
	if (length != 1) return (LENGTH_ERROR);			   /* Check length		 */
	return (NO_ERROR);								   /* Return OK		 */
}

int ProcessJntPos(BYTE *buf, int len, float *time, float jp[6]){
	int i;
	BYTE *cPtr;

	if (len != 7 * 4) {
		return (LENGTH_ERROR); /* Check length	   */
	}

	cPtr =(BYTE *)time;
	for (i = 0; i < 4; i++) { /* Copy incoming data	 */
		*(cPtr + i) = buf[i];
	}
	C4x_To_IEEE_32(time, 1); /* Convert data	 */

	cPtr =(BYTE *)jp;
	for (i = 0; i < 6 * 4; i++) { /* Copy incoming data	 */
		*(cPtr + i) = buf[i + 4];
	}
	C4x_To_IEEE_32(jp, 6); /* Convert data	 */
	return (NO_ERROR);	   /* Return OK		 */
}

/*****************************************************************
* Function: HandStatus
*
* Returns: 
*		0 if no error
*		1 if error
*		connected is set to 
*			- 1 if hand controller is present.
*			- 0 if hand controller is absent.
******************************************************************/
int HandStatus(int *connected){
	int retval;

	OutBuf[0] = HAND_STATUS; /* Write command code	 */
	if ((retval = Send(OutBuf, 1)) != 0) { /* Send Command		 */
		if (retval == 2) return (SEND_ERROR);
		else return (SEND_LENGTH_ERROR);   /* retval must be == 1 */
	}

	if ((length = Receive(InBuf)) == 0) return (RECEIVE_ERROR);
	if (InBuf[0] != HAND_STATUS) return (InBuf[0]);	/* Check code */
	if (length != 2) return (LENGTH_ERROR);			/* Check length		 */
	if (InBuf[1] == 0) 
		*connected=1;
	else	
		*connected=0;
	return (0);
	 
}


/*****************************************************************
* Function: FootStatus
*
* Returns: 
*		0 if no error
*		1 if error
*		connected is set to 
*			- 1 if foot controller is present.
*			- 0 if foot controller is absent.
******************************************************************/
int FootStatus(int *connected){
	int retval;

	OutBuf[0] = FOOT_STATUS; /* Write command code	 */
	if ((retval = Send(OutBuf, 1)) != 0) { /* Send Command		 */
		if (retval == 2) return (SEND_ERROR);
		else return (SEND_LENGTH_ERROR);   /* retval must be == 1 */
	}

	if ((length = Receive(InBuf)) == 0) return (RECEIVE_ERROR);
	if (InBuf[0] != FOOT_STATUS) return (InBuf[0]);	/* Check code */
	if (length != 2) return (LENGTH_ERROR);			/* Check length		 */
	if (InBuf[1] == 0) 
		*connected=1;
	else
		*connected=0;
	
	return (0);
}


/*****************************************************************
* Function: Servo
*	Start and stop servo on the named joint
* Returns: 0 if OK
*	   1 if error
******************************************************************/
int Servo(int joint, BYTE command[]){
	int retval;

	OutBuf[0] = SERVO;				/* Write command code	 */
	OutBuf[1] = joint;				/* Write joint number	 */
	if (strcmp(command, "on") == 0)	/* Test command	 */
		OutBuf[2] = 0;				/* Write command code	  */
	else OutBuf[2] = 1;
	if ((retval = Send(OutBuf, 3)) != 0) { /* Send Command		 */
		if (retval == 2) return (SEND_ERROR);
		else return (SEND_LENGTH_ERROR);   /* retval must be == 1 */
	}

	if ((length = Receive(InBuf)) == 0) return (RECEIVE_ERROR);
	if (InBuf[0] != SERVO) return (InBuf[0]); /* Check code */
	if (length != 1) return (LENGTH_ERROR);	  /* Check length		 */
	return (NO_ERROR);
}


/*****************************************************************
* Function: Brake
*	Control brake for joints 1 and 2
* Returns: 0 if  OK
*	   1 if error
******************************************************************/
int Brake(BYTE command[]){
	int retval;

	OutBuf[0] = BRAKE;				/* Write command code	 */
	if (strcmp(command, "on") == 0)	/* Test command	 */
		OutBuf[1] = 1;				/* Write command code	  */
	else 
		OutBuf[1] = 0;

	if ((retval = Send(OutBuf, 2)) != 0) { /* Send Command		 */
		if (retval == 2) 
			return (SEND_ERROR);
		else 
			return (SEND_LENGTH_ERROR);   /* retval must be == 1 */
	}

	if ((length = Receive(InBuf)) == 0) 
		return (RECEIVE_ERROR);
	if (InBuf[0] != BRAKE) 
		return (InBuf[0]); /* Check code */
	if (length != 1) 
		return (LENGTH_ERROR);	  /* Check length		 */

	return (NO_ERROR);
}




/*****************************************************************
* Function: PlayVFBMessage
*	Request a VFB message
* Returns:	0 if  OK
*			1 if error
******************************************************************/
int PlayVFBMessage(BYTE type){
	int retval;

	OutBuf[0] = PLAY_VFB; /* Write command code	 */
	OutBuf[1] = type;
	if ((retval = Send(OutBuf, 2)) != 0) { /* Send Command		 */
		if (retval == 2) 
			return (SEND_ERROR);
		else 
			return (SEND_LENGTH_ERROR);   /* retval must be == 1 */
	}

	if ((length = Receive(InBuf)) == 0) 
		return (RECEIVE_ERROR);
	if (InBuf[0] != PLAY_VFB) 
		return (InBuf[0]); /* Check code */
	if (length != 1) 
		return (LENGTH_ERROR);	  /* Check length		 */
	return (NO_ERROR);
}


/*****************************************************************
* Function: SetZLimit
*	Set the lower z limit.
* Returns: 0 if  OK
*	   1 if error
******************************************************************/
int SetZLimit(BYTE reset){
	int retval;

	OutBuf[0] = SET_ZLIMIT;	/* Write command code	 */
	OutBuf[1] = reset;
	if ((retval = Send(OutBuf, 2)) != 0) { /* Send Command		 */
		if (retval == 2) return (SEND_ERROR);
		else return (SEND_LENGTH_ERROR);   /* retval must be == 1 */
	}

	if ((length = Receive(InBuf)) == 0) return (RECEIVE_ERROR);
	if (InBuf[0] != SET_ZLIMIT) return (InBuf[0]); /* Check code */
	if (length != 1) return (LENGTH_ERROR);		   /* Check length		 */
	return (NO_ERROR);
}

/*****************************************************************
* Function: AesopStatus
*
* Returns: 0 if successful.
*	   non-zero otherwise
******************************************************************/
int AesopStatus(BYTE *robotStat, BYTE *last_command, BYTE *last_cmdarg){
	int retval;

	OutBuf[0] = AESOP_STATUS; /* Write command code	 */
	if ((retval = Send(OutBuf, 1)) != 0) { /* Send Command		 */
		if (retval == 2) 
				return (SEND_ERROR);
		else 
			return (SEND_LENGTH_ERROR);   /* retval must be == 1 */
	}

	if ((length = Receive(InBuf)) == 0) 
		return (RECEIVE_ERROR);
	if (InBuf[0] != AESOP_STATUS) 
		return (InBuf[0]);	/* Check code */
	if (length != 4)		/* Check length		 */
		return (LENGTH_ERROR);
			 
	*robotStat = InBuf[1];
	*last_command = InBuf[2];
	*last_cmdarg = InBuf[3];
	
	return (NO_ERROR);
}

/*****************************************************************
* Function: Home
*	Have the robot move to home position
* Returns: 0 if  OK
*	   1 if error
******************************************************************/
int Home(){
	int retval;

	OutBuf[0] = HOME; /* Write command code	 */
	if ((retval = Send(OutBuf, 1)) != 0) { /* Send Command		 */
		if (retval == 2) return (SEND_ERROR);
		else return (SEND_LENGTH_ERROR);   /* retval must be == 1 */
	}

	if ((length = Receive(InBuf)) == 0) return (RECEIVE_ERROR);
	if (InBuf[0] != HOME) return (InBuf[0]); /* Check code */
	if (length != 1) return (LENGTH_ERROR);	 /* Check length		 */
	return (NO_ERROR);
}


/*****************************************************************
* Function: MoveUp
*	Have the robot pan up relative to the screen
* Returns: 0 if  OK
*	   1 if error
******************************************************************/
int MoveUp(BYTE speed){
	int retval;

	OutBuf[0] = MOVE_UP; /* Write command code	 */
	OutBuf[1] = speed;	 /* Write parameter	 */
	if ((retval = Send(OutBuf, 2)) != 0) { /* Send Command		 */
		if (retval == 2) return (SEND_ERROR);
		else return (SEND_LENGTH_ERROR);   /* retval must be == 1 */
	}

	if ((length = Receive(InBuf)) == 0) return (RECEIVE_ERROR);
	if (InBuf[0] != MOVE_UP) return (InBuf[0]);	/* Check code */
	if (length != 1) return (LENGTH_ERROR);		/* Check length		 */
	return (NO_ERROR);
}


/*****************************************************************
* Function: MoveDown
*	Have the robot pan down relative to the screen
* Returns: 0 if  OK
*	   1 if error
******************************************************************/
int MoveDown(BYTE speed){
	int retval;

	OutBuf[0] = MOVE_DOWN; /* Write command code	 */
	OutBuf[1] = speed;	   /* Write parameter	 */
	if ((retval = Send(OutBuf, 2)) != 0) { /* Send Command		 */
		if (retval == 2) return (SEND_ERROR);
		else return (SEND_LENGTH_ERROR);   /* retval must be == 1 */
	}

	if ((length = Receive(InBuf)) == 0) return (RECEIVE_ERROR);
	if (InBuf[0] != MOVE_DOWN) return (InBuf[0]); /* Check code */
	if (length != 1) return (LENGTH_ERROR);		  /* Check length		 */
	return (NO_ERROR);
}


/*****************************************************************
* Function: MoveRight
*	Have the robot pan right relative to the screen
* Returns: 0 if  OK
*	   1 if error
******************************************************************/
int MoveRight(BYTE speed){
	int retval;

	OutBuf[0] = MOVE_RIGHT;	/* Write command code	 */
	OutBuf[1] = speed;		/* Write parameter	 */
	if ((retval = Send(OutBuf, 2)) != 0) { /* Send Command		 */
		if (retval == 2) return (SEND_ERROR);
		else return (SEND_LENGTH_ERROR);   /* retval must be == 1 */
	}

	if ((length = Receive(InBuf)) == 0) return (RECEIVE_ERROR);
	if (InBuf[0] != MOVE_RIGHT) return (InBuf[0]); /* Check code */
	if (length != 1) return (LENGTH_ERROR);		   /* Check length		 */
	return (NO_ERROR);
}


/*****************************************************************
* Function: MoveLeft
*	Have the robot pan left relative to the screen
* Returns: 0 if  OK
*	   1 if error
******************************************************************/
int MoveLeft(BYTE speed){
	int retval;

	OutBuf[0] = MOVE_LEFT; /* Write command code	 */
	OutBuf[1] = speed;	   /* Write parameter	 */
	if ((retval = Send(OutBuf, 2)) != 0) { /* Send Command		 */
		if (retval == 2) return (SEND_ERROR);
		else return (SEND_LENGTH_ERROR);   /* retval must be == 1 */
	}

	if ((length = Receive(InBuf)) == 0) return (RECEIVE_ERROR);
	if (InBuf[0] != MOVE_LEFT) return (InBuf[0]); /* Check code */
	if (length != 1) return (LENGTH_ERROR);		  /* Check length		 */
	return (NO_ERROR);
}


/*****************************************************************
* Function: ZoomIn
*	Have the robot zoom in relative to the screen
* Returns: 0 if  OK
*	   1 if error
******************************************************************/
int ZoomIn(BYTE speed){
	int retval;

	OutBuf[0] = ZOOM_IN; /* Write command code	 */
	OutBuf[1] = speed;	 /* Write parameter	 */
	if ((retval = Send(OutBuf, 2)) != 0) { /* Send Command		 */
		if (retval == 2) return (SEND_ERROR);
		else return (SEND_LENGTH_ERROR);   /* retval must be == 1 */
	}

	if ((length = Receive(InBuf)) == 0) return (RECEIVE_ERROR);
	if (InBuf[0] != ZOOM_IN) return (InBuf[0]);	/* Check code */
	if (length != 1) return (LENGTH_ERROR);		/* Check length		 */
	return (NO_ERROR);
}


/*****************************************************************
* Function: ZoomOut
*	Have the robot zoom out relative to the screen
* Returns: 0 if  OK
*	   1 if error
******************************************************************/
int ZoomOut(BYTE speed){
	int retval;

	OutBuf[0] = ZOOM_OUT; /* Write command code	 */
	OutBuf[1] = speed;	  /* Write parameter	 */
	if ((retval = Send(OutBuf, 2)) != 0) { /* Send Command		 */
		if (retval == 2) return (SEND_ERROR);
		else return (SEND_LENGTH_ERROR);   /* retval must be == 1 */
	}

	if ((length = Receive(InBuf)) == 0) return (RECEIVE_ERROR);
	if (InBuf[0] != ZOOM_OUT) return (InBuf[0]); /* Check code */
	if (length != 1) return (LENGTH_ERROR);		 /* Check length		 */
	return (NO_ERROR);
}


/*****************************************************************
* Function: Stop
*	Stop the robot
* Returns: 0 if  OK
*	   1 if error
******************************************************************/
int Stop(){
	int retval;

	OutBuf[0] = STOP; /* Write command code	 */
	if ((retval = Send(OutBuf, 1)) != 0) { /* Send Command		 */
		if (retval == 2) return (SEND_ERROR);
		else return (SEND_LENGTH_ERROR);   /* retval must be == 1 */
	}

	if ((length = Receive(InBuf)) == 0) return (RECEIVE_ERROR);
	if (InBuf[0] != STOP) return (InBuf[0]); /* Check code */
	if (length != 1) return (LENGTH_ERROR);	 /* Check length		 */
	return (NO_ERROR);
}


/*****************************************************************
* Function: Start
*	Start the robot
* Returns: 0 if  OK
*	   1 if error
******************************************************************/
int Start(){
	int retval;

	OutBuf[0] = START; /* Write command code	 */
	if ((retval = Send(OutBuf, 1)) != 0) { /* Send Command		 */
		if (retval == 2) return (SEND_ERROR);
		else return (SEND_LENGTH_ERROR);   /* retval must be == 1 */
	}

	if ((length = Receive(InBuf)) == 0) return (RECEIVE_ERROR);
	if (InBuf[0] != START) return (InBuf[0]); /* Check code */
	if (length != 1) return (LENGTH_ERROR);	  /* Check length		 */
	return (NO_ERROR);
}


/*****************************************************************
* Function: Clear
*	Clear the motion queue of the robot
* Returns: 0 if  OK
*	   1 if error
******************************************************************/
int Clear(){
	int retval;

	OutBuf[0] = CLEAR; /* Write command code	 */
	if ((retval = Send(OutBuf, 1)) != 0) { /* Send Command		 */
		if (retval == 2) return (SEND_ERROR);
		else return (SEND_LENGTH_ERROR);   /* retval must be == 1 */
	}

	if ((length = Receive(InBuf)) == 0) return (RECEIVE_ERROR);
	if (InBuf[0] != CLEAR) return (InBuf[0]); /* Check code */
	if (length != 1) return (LENGTH_ERROR);	  /* Check length		 */
	return (NO_ERROR);
}


/*****************************************************************
* Function: MoveJntRel
*	Move the given joint relative to where it is.
* Returns: 0 if  OK
*	   1 if error
******************************************************************/
int MoveJntRel(BYTE joint, float delta){
	int retval;

	OutBuf[0] = MOVE_REL;		/* Write command code	 */
	OutBuf[1] = joint;			/* Write joint number	 */
	PacFloat(OutBuf, 2, delta);	/* Load delta angle */
	IEEE_To_C4x_32(&OutBuf[2], 1);
	if ((retval = Send(OutBuf, 6)) != 0) { /* Send Command		 */
		if (retval == 2) return (SEND_ERROR);
		else return (SEND_LENGTH_ERROR);   /* retval must be == 1 */
	}

	if ((length = Receive(InBuf)) == 0) return (RECEIVE_ERROR);
	if (InBuf[0] != MOVE_REL) return (InBuf[0]); /* Check code */
	if (length != 1) return (LENGTH_ERROR);		 /* Check length		 */
	return (NO_ERROR);
}


/*****************************************************************
* Function: MoveJntAbs
*	Move the given joint to a absolute position.
* Returns: 0 if  OK
*	   1 if error
******************************************************************/
int MoveJntAbs(BYTE joint, float angle){
	int retval;

	OutBuf[0] = MOVE_ABS;		/* Write command code	 */
	OutBuf[1] = joint;			/* Write joint number	 */
	PacFloat(OutBuf, 2, angle);	/* Load delta angle */
	IEEE_To_C4x_32(&OutBuf[2], 1);
	if ((retval = Send(OutBuf, 6)) != 0) { /* Send Command		 */
		if (retval == 2) return (SEND_ERROR);
		else return (SEND_LENGTH_ERROR);   /* retval must be == 1 */
	}

	if ((length = Receive(InBuf)) == 0) return (RECEIVE_ERROR);
	if (InBuf[0] != MOVE_ABS) return (InBuf[0]); /* Check code */
	if (length != 1) return (LENGTH_ERROR);		 /* Check length		 */
	return (NO_ERROR);
}

#ifdef OLD_SCHOOL
/*****************************************************************
* Function: MoveJointContinous
*	Move the given joint with a given speed.
* Returns: 0 if  OK
*	   1 if error
******************************************************************/
int MoveJointContinous(BYTE joint, float speed){
	int retval;

	OutBuf[0] = MOVE_JOINT_VEL;		/* Write command code	 */
	OutBuf[1] = joint;				/* Write joint number	 */
	PacFloat(OutBuf, 2, speed);		/* Load speed */
	IEEE_To_C4x_32(&OutBuf[2], 1);
	if ((retval = Send(OutBuf, 6)) != 0) { /* Send Command		 */
		if (retval == 2) 
			return (SEND_ERROR);
		else 
			return (SEND_LENGTH_ERROR);   /* retval must be == 1 */
	}

	if ((length = Receive(InBuf)) == 0) 
		return (RECEIVE_ERROR);
	if (InBuf[0] != MOVE_JOINT_VEL) 
		return (InBuf[0]); /* Check code */
	if (length != 1) 
		return (LENGTH_ERROR);		 /* Check length		 */
	return (NO_ERROR);
}

#endif // OLD_SCHOOL


/*****************************************************************
* Function: MoveJointContinous
*	Move joints 0,1,2,3 with given speeds.
* Returns: 0 if  OK
*	   1 if error
******************************************************************/
int MoveAllJointsContinous(float *speed){
	int retval, i;

	OutBuf[0] = MOVE_ALLJOINTS_VEL;		/* Write command code	 */
	
	for(i=0;i<4;i++) {
		PacFloat(OutBuf, i*4+1, speed[i]);		/* Load speeds */
		IEEE_To_C4x_32(&OutBuf[i*4+1], 1);	
	}

	if ((retval = Send(OutBuf, 4*4+1)) != 0) { /* Send Command		 */
		if (retval == 2) 
			return (SEND_ERROR);
		else 
			return (SEND_LENGTH_ERROR);   /* retval must be == 1 */
	}

	if ((length = Receive(InBuf)) == 0) 
		return (RECEIVE_ERROR);
	if (InBuf[0] != MOVE_ALLJOINTS_VEL) 
		return (InBuf[0]); /* Check code */
	if (length != 1) 
		return (LENGTH_ERROR);		 /* Check length		 */
	return (NO_ERROR);
}










/*****************************************************************
* Function: MoveTipRel
*	Move the tip relative to its location.
* Returns: 0 if  OK
*	   1 if error
******************************************************************/
int MoveTipRel(float vector[3], float mag){
	int retval;

	OutBuf[0] = MOVE_TIP_REL;		/* Write command code	 */
	PacFloat(OutBuf, 1, vector[0]);	/* Load x	  */
	PacFloat(OutBuf, 5, vector[1]);	/* Load y	  */
	PacFloat(OutBuf, 9, vector[2]);	/* Load z	  */
	PacFloat(OutBuf, 13, mag);		/* Load magnitude	 */
	IEEE_To_C4x_32(&OutBuf[1], 4);
	if ((retval = Send(OutBuf, 17)) != 0) {	/* Send Command	 */
		if (retval == 2) 
			return (SEND_ERROR);
		else 
			return (SEND_LENGTH_ERROR);	/* retval must be == 1 */
	}

	if ((length = Receive(InBuf)) == 0) return (RECEIVE_ERROR);
	if (InBuf[0] != MOVE_TIP_REL) return (InBuf[0]); /* Check code */
	if (length != 1) return (LENGTH_ERROR);			 /* Check length		 */
	return (NO_ERROR);
}


/*****************************************************************
* Function: MoveTipAbs
*	Move the tip to a absolute position.
* Returns: 0 if  OK
*	   1 if error
******************************************************************/
int MoveTipAbs(float position[3]){
	int retval;

	OutBuf[0] = MOVE_TIP_ABS;		  /* Write command code	 */
	PacFloat(OutBuf, 1, position[0]); /* Load x	  */
	PacFloat(OutBuf, 5, position[1]); /* Load y	  */
	PacFloat(OutBuf, 9, position[2]); /* Load z	  */
	IEEE_To_C4x_32(&OutBuf[1], 3);
	if ((retval = Send(OutBuf, 13)) != 0) {	/* Send Command	 */
		if (retval == 2) return (SEND_ERROR);
		else return (SEND_LENGTH_ERROR);	/* retval must be == 1 */
	}

	if ((length = Receive(InBuf)) == 0) return (RECEIVE_ERROR);
	if (InBuf[0] != MOVE_TIP_ABS) return (InBuf[0]); /* Check code */
	if (length != 1) return (LENGTH_ERROR);			 /* Check length		 */
	return (NO_ERROR);
}


/*****************************************************************
* Function: MemRead
*	Read the tip position and orientation stored memory.
* Returns: 0 if  OK
*	   1 if error
******************************************************************/
int MemRead(BYTE mem, float position[6]){
	int retval;

	OutBuf[0] = MEM_READ; /* Write command code	 */
	OutBuf[1] = mem;	  /* Write command code	 */
	if ((retval = Send(OutBuf, 2)) != 0) { /* Send Command	 */
		if (retval == 2) return (SEND_ERROR);
		else return (SEND_LENGTH_ERROR);   /* retval must be == 1 */
	}

	if ((length = Receive(InBuf)) == 0) return (RECEIVE_ERROR);
	if (InBuf[0] != MEM_READ) return (InBuf[0]);	/* Check code	 */
	if (length != 6 * 4 + 1) return (LENGTH_ERROR);	/* Check length	 */
	UnpacFloat(&position[0], InBuf, 1);				/* convert x position	 */
	UnpacFloat(&position[1], InBuf, 5);				/* convert y position	 */
	UnpacFloat(&position[2], InBuf, 9);				/* convert z position	 */
	UnpacFloat(&position[3], InBuf, 13);			/* convert phi		 */
	UnpacFloat(&position[4], InBuf, 17);			/* convert theta	 */
	UnpacFloat(&position[5], InBuf, 21);			/* convert psi		 */
	C4x_To_IEEE_32(&position[0], 6);				/* Convert data	 */
	return (NO_ERROR);
}


/*****************************************************************
* Function: MemSave
*	Saves the current tip position in one of six memory locations.
* Returns: 0 if  OK
*	   1 if error
******************************************************************/
int MemSave(BYTE mem){
	int retval;

	OutBuf[0] = MEM_SAVE; /* Write command code	 */
	OutBuf[1] = mem;	  /* Write command code	 */
	if ((retval = Send(OutBuf, 2)) != 0) { /* Send Command	 */
		if (retval == 2) return (SEND_ERROR);
		else return (SEND_LENGTH_ERROR);   /* retval must be == 1 */
	}

	if ((length = Receive(InBuf)) == 0) return (RECEIVE_ERROR);
	if (InBuf[0] != MEM_SAVE) return (InBuf[0]); /* Check code	 */
	if (length != 1) return (LENGTH_ERROR);		 /* Check length		 */
	return (NO_ERROR);
}


/*****************************************************************
* Function: MemReturn
*	Return the tip to the position in memory.
* Returns: 0 if  OK
*	   1 if error
******************************************************************/
int MemReturn(BYTE mem){
	int retval;

	OutBuf[0] = MEM_RETURN;	/* Write command code	 */
	OutBuf[1] = mem;		/* Write command code	 */
	if ((retval = Send(OutBuf, 2)) != 0) { /* Send Command	 */
		if (retval == 2) return (SEND_ERROR);
		else return (SEND_LENGTH_ERROR);   /* retval must be == 1 */
	}

	if ((length = Receive(InBuf)) == 0) return (RECEIVE_ERROR);
	if (InBuf[0] != MEM_RETURN) return (InBuf[0]); /* Check code */
	if (length != 1) return (LENGTH_ERROR);		   /* Check length		 */
	return (NO_ERROR);
}


/*****************************************************************
* Function: JntTrajLSPB
*	Define the trajectory of a joint.
*
* Parameters
*	joint:	0-5
*	posI = start point
*	posF = end point
*	vel = velocity
*	acc = accleration
*	time = duration of move
*
* Returns: 0 if  OK
*	   1 if error
int JntTrajLSPB(BYTE joint, float posI, float posF, float vel, float acc, float time)
******************************************************************/
int JntTrajLSPB(BYTE joint, float posF, float vel, float acc){
	int retval;

	OutBuf[0] = JNT_TRAJ_LSPB;	   /* Write command code	 */
	OutBuf[1] = joint;			   /* Write joint number	 */
	PacFloat(OutBuf, 2, posF);	   /* Write end position	 */
	PacFloat(OutBuf, 6, vel);	   /* Write velocity	 */
	PacFloat(OutBuf, 10, acc);	   /* Write acceleration	 */
	IEEE_To_C4x_32(&OutBuf[2], 3); /* Convert data	 */
	if ((retval = Send(OutBuf, 14)) != 0) {	/* Send Command	 */
		if (retval == 2) return (SEND_ERROR);
		else return (SEND_LENGTH_ERROR);	/* retval must be == 1 */
	}

	if ((length = Receive(InBuf)) == 0) 
		return (RECEIVE_ERROR);
	if (InBuf[0] != JNT_TRAJ_LSPB) 
		return (InBuf[0]); /* Check code */
	if (length != 1) 
		return (LENGTH_ERROR);			  /* Check length		 */
	return (NO_ERROR);
}


/*****************************************************************
* Function: JntTrajCS
*	Set up a trajectory using the cubic spline method.
*
* Parameters
*	joint:	0-5
*	posI = Initial position
*	posF = Final position
*	velI = Initial velocity
*	velF = Final velocity
*	time = Duration of move
*
* Returns: 0 if  OK
*	   1 if error
int JntTrajCS(BYTE joint, float posI, float posF, float velI, float velF, float time)
******************************************************************/
int JntTrajCS(BYTE joint, float posF, float velF){
	int retval;

	OutBuf[0] = JNT_TRAJ_CS;	   /* Write command code	 */
	OutBuf[1] = joint;			   /* Write joint number	 */
	PacFloat(OutBuf, 2, posF);	   /* Write end position	 */
	PacFloat(OutBuf, 6, velF);	   /* Write acceleration	 */
	IEEE_To_C4x_32(&OutBuf[2], 2); /* Convert data	 */
	if ((retval = Send(OutBuf, 10)) != 0) {	/* Send Command	 */
		if (retval == 2) return (SEND_ERROR);
		else return (SEND_LENGTH_ERROR);	/* retval must be == 1 */
	}

	if ((length = Receive(InBuf)) == 0) return (RECEIVE_ERROR);
	if (InBuf[0] != JNT_TRAJ_CS) return (InBuf[0]);	/* Check code */
	if (length != 1) return (LENGTH_ERROR);			/* Check length		 */
	return (NO_ERROR);
}


/*****************************************************************
* Function: TipTrajStep
*	Define the trajectory of the tip.
*	This uses the Step trajectory planner.
* Parameters
*	posI = start point
*	posF = final point
* Returns: 0 if  OK
*	   1 if error
******************************************************************/
int TipTrajStep(float posI[3], float posF[3]){
	int retval;

	OutBuf[0] = TIP_TRAJ_STEP;	   /* Write command code	 */
	PacFloat(OutBuf, 1, posI[0]);  /* Write start x	  */
	PacFloat(OutBuf, 5, posI[1]);  /* Write start y	  */
	PacFloat(OutBuf, 9, posI[2]);  /* Write start z	  */
	PacFloat(OutBuf, 13, posF[0]); /* Write final x	  */
	PacFloat(OutBuf, 17, posF[1]); /* Write final y	  */
	PacFloat(OutBuf, 21, posF[2]); /* Write final z	  */
	IEEE_To_C4x_32(&OutBuf[1], 6); /* Convert data	 */
	if ((retval = Send(OutBuf, 25)) != 0) {	/* Send Command	 */
		if (retval == 2) return (SEND_ERROR);
		else return (SEND_LENGTH_ERROR);	/* retval must be == 1 */
	}

	if ((length = Receive(InBuf)) == 0) return (RECEIVE_ERROR);
	if (InBuf[0] != TIP_TRAJ_STEP) return (InBuf[0]); /* Check code */
	if (length != 1) return (LENGTH_ERROR);			  /* Check length		 */
	return (NO_ERROR);
}


/*****************************************************************
* Function: TipTrajLSPB
*	Set up a tip trajectory using Linear Segment Parobolic Blend
*
* Parameters
*	posI = start point
*	posF = final point
*	vel  = velocity
*	acc  = accleration
*	time = duration of move
*
* Returns: 0 if  OK
*	   1 if error
int TipTrajLSPB(float posI[3], float posF[3], float vel, float acc, float time)
******************************************************************/
int TipTrajLSPB(float posF[3], float vel, float acc){
	int retval;

	OutBuf[0] = TIP_TRAJ_LSPB;	   /* Write command code	 */
	PacFloat(OutBuf, 1, posF[0]);  /* Write final x	  */
	PacFloat(OutBuf, 5, posF[1]);  /* Write final y	  */
	PacFloat(OutBuf, 9, posF[2]);  /* Write final z	  */
	PacFloat(OutBuf, 13, vel);	   /* Write velocity	  */
	PacFloat(OutBuf, 17, acc);	   /* Write acceleration	  */
	IEEE_To_C4x_32(&OutBuf[1], 5); /* Convert data	 */
	if ((retval = Send(OutBuf, 21)) != 0) {	/* Send Command	 */
		if (retval == 2) return (SEND_ERROR);
		else return (SEND_LENGTH_ERROR);	/* retval must be == 1 */
	}

	if ((length = Receive(InBuf)) == 0) return (RECEIVE_ERROR);
	if (InBuf[0] != TIP_TRAJ_LSPB) return (InBuf[0]); /* Check code */
	if (length != 1) return (LENGTH_ERROR);			  /* Check length		 */
	return (NO_ERROR);
}


/*****************************************************************
* Function: TipTrajCS
*	Set up a tip trajectory using the cubic spline method.
*
* Parameters
*	posI = Initial position
*	posF = Final position
*	velI = Initial velocity
*	velF = Final velocity
*	time = Duration of move
*
* Returns: 0 if  OK
*	   1 if error
int TipTrajCS(float posI, float posF, float velI, float velF, float time)
******************************************************************/
int TipTrajCS(float posF[3]){
	int retval;

	OutBuf[0] = TIP_TRAJ_CS;	   /* Write command code	 */
	PacFloat(OutBuf, 1, posF[0]);  /* Write end position	 */
	PacFloat(OutBuf, 5, posF[1]);  /* Write end position	 */
	PacFloat(OutBuf, 9, posF[2]);  /* Write end velocity	 */
	IEEE_To_C4x_32(&OutBuf[1], 3); /* Convert data	 */
	if ((retval = Send(OutBuf, 13)) != 0) {	/* Send Command	 */
		if (retval == 2) return (SEND_ERROR);
		else return (SEND_LENGTH_ERROR);	/* retval must be == 1 */
	}

	if ((length = Receive(InBuf)) == 0) return (RECEIVE_ERROR);
	if (InBuf[0] != TIP_TRAJ_CS) return (InBuf[0]);	/* Check code */
	if (length != 1) return (LENGTH_ERROR);			/* Check length		 */
	return (NO_ERROR);
}


/*****************************************************************
* Function: UARTread
*****************************************************************/
int UARTread(BYTE regs[7]){
	int retval;
	int i;

	OutBuf[0] = UART_REGS; /* Write command code	 */
	if ((retval = Send(OutBuf, 1)) != 0) { /* Send Command		 */
		if (retval == 2) 
			return (SEND_ERROR);
		else 
			return (SEND_LENGTH_ERROR);   /* retval must be == 1 */
	}

	if ((length = Receive(InBuf)) == 0) 
		return (RECEIVE_ERROR);
	if (InBuf[0] != UART_REGS) 
		return (InBuf[0]); /* Check code */
	if (length != 9) {
		printf("UART received length = %d\n", length);
		return (LENGTH_ERROR); /* Check length	   */
	}
	for (i = 0; i < 8; i++) {
		regs[i] = InBuf[i + 1];
	}
	return (NO_ERROR);
}

/*****************************************************************
* Function: Execute
*****************************************************************/
int Execute(long addr){
	int retval;

	OutBuf[0] = EXECUTE; /* Write command code	 */
	*(long *)&OutBuf[1] = addr;
	if ((retval = Send(OutBuf, 5)) != 0) { /* Send Command	 */
		if (retval == 2) 
			return (SEND_ERROR);
		else 
			return (SEND_LENGTH_ERROR);   /* retval must be == 1 */
	}

	if ((length = Receive(InBuf)) == 0) 
		return (RECEIVE_ERROR);
	if (InBuf[0] != EXECUTE) 
		return (InBuf[0]);	/* Check code */
	if (length != 1) 
		return (LENGTH_ERROR);		/* Check length		 */
	return (NO_ERROR);
}

/* ---------------- local data ----------------- */
static unsigned int filesize; /* num of blks */
static unsigned BYTE bf[130]; /* i/o buffer	 */
static FILE *fp;


int Download(unsigned long *execAddr, int execflag, BYTE *fname) {
	int retval=0;

	// Display Transfer Status Bar
	printf("<Download Requested>\n");

	// Open File
	if ((fp = fopen(fname, "rb")) == NULL) {
		printf("\nNo such file as \"%s\"\n", fname);
		return DLOAD_ERROR;
	}

	// Send download command 
	OutBuf[0] = DOWNLOAD;
	OutBuf[1] = execflag;
	if ((retval = Send(OutBuf, 2)) != 0) { /* Send Command	*/
		if (retval == 2) 
			return (SEND_ERROR);
		else 
			return (SEND_LENGTH_ERROR);   /* retval must be == 1 */
	}

	// Receive Download Confirmation
	if ((length = Receive(InBuf)) == 0) 
		return (RECEIVE_ERROR);
	if (InBuf[0] != DOWNLOAD)
		return (InBuf[0]);

	// Start Downloading
	retval = send_cmi(fp, execAddr);

	// Close File
	fclose(fp);
	
	return (retval);
}

// Perform XModem Download
#define	STARS		70
#define BLOCK_SIZE	128
int send_cmi(FILE* fp, unsigned long *execAddr) {
	register int i;
	int endoffile = FALSE, ln;
	long blocks =0;
	long block = 0;
	int retval;
	
	// Compute Number of Blocks
	blocks=filelength(fileno(fp))/BLOCK_SIZE+1;
	printf("<Blocks=%ld>\n",blocks);

	// Send 1 Block
	while (!endoffile) {

		// Read Data Block From File
		if ((ln = fread(&OutBuf[1], 1, BLOCK_SIZE, fp))!=BLOCK_SIZE) 
			endoffile = TRUE;
		if (ln == 0) 
			break;

		// User Cancel?
		if (kbhit() && getch() == 033) {
			printf("<Download CANCELLED>\n");
			
			// Send Cancel
			OutBuf[0] = CANCEL_COMMAND;
			if ((retval = Send(OutBuf, 1)) != 0) { 
				if (retval == 2) 
					return (SEND_ERROR);
				else 
					return (SEND_LENGTH_ERROR);
			}

			// Receive Cancel Confirmation
			if ((length = Receive(InBuf)) == 0) 
				return (RECEIVE_ERROR);
			if (InBuf[0] != CANCEL_COMMAND) 
				return (InBuf[0]);

			return CANCEL_COMMAND;
		}

		// Send Data Block
		OutBuf[0] = DLOAD_DATA;
		if ((retval = Send(OutBuf, BLOCK_SIZE+1)) != 0) {
			if (retval == 2) 
				return (SEND_ERROR);
			else 
				return (SEND_LENGTH_ERROR);
		}

		// Receive Data Confirmation
		if ((length = Receive(InBuf)) == 0) 
			return (RECEIVE_ERROR);
		if (InBuf[0] != DLOAD_DATA) {
			return (InBuf[0]);
		}

		// Update Transfer Status Display
 		block++;
 		if(((block-1)%(blocks/STARS))==0)
			printf("#");
	}
	
	// Send End Of Transmit
	OutBuf[0] = DLOAD_END;
	if ((retval = Send(OutBuf, 2)) != 0) {
		if (retval == 2) 
			return (SEND_ERROR);
		else 
			return (SEND_LENGTH_ERROR);   
	}

	// Receive End of Transmit Confirmation
	if ((length = Receive(InBuf)) == 0) 
		return (RECEIVE_ERROR);
	if (length != 5) 
		return (LENGTH_ERROR);
	if (InBuf[0] != DLOAD_END)
		return (InBuf[0]);

	printf("\n<Download Complete>\n");

	// Set Execute Address
	*execAddr = *(unsigned long*) &InBuf[1];
	
	return (0xff & NO_ERROR);
}



void PrintRobotStat(int rstat, int lc, int lcarg) {
#define START_LINE 10

	DisplayGoto(2,START_LINE);
	printf("Robot Status:");
	
	DisplayGoto(2,START_LINE +2);
	if(rstat & M_SCOPE) 
		printf("CONTROLS:         -enabled");
	else
		printf("CONTROLS:         -disabled");

	DisplayGoto(2,START_LINE +3);
	if(rstat & M_ZLIMIT)
		printf("LOWERLIMIT:       -set");
	else
		printf("LOWERLIMIT:       -cleared");

	DisplayGoto(2,START_LINE +4);
	if(rstat & M_DISABLE)
		printf("MANUAL MODE:      -enabled");
	else
		printf("MANUAl MODE:      -disabled");

	DisplayGoto(2,START_LINE +5);
	if(rstat & M_HAND)
		printf("INPUTS:           -hand controller connected");
	else if(rstat & M_FOOT)
		printf("INPUTS:           -foot controller connected");
	else
		printf("INPUTS:           -none");

	DisplayGoto(2,START_LINE +6);
	if(lc)
		printf("LAST COMMAND:     -0x%x [%d]", lc, lcarg);


	return;
}