#ifndef A3000_H
#define A3000_H
/********************************************************
*							*
*  AESOP Development System				*
*							*
*  AESOP Commands header file				*
*							*
*  Author:  A. A. Mangaser				*
*							*
*  Version: $Revision:   1.0  $					*
*							*
*  Copyright (C) 1994 Computer Motion Inc.		*
*********************************************************/

#include "Iol.hpp"
#include "Hp_cs.hpp"

/*********************************************************************
* This collections of functions implement the AESOP command set
*********************************************************************/

#define ON	0
#define OFF	1

/* Masks for stat regs */
#define M_SCOPE		(BYTE)0x01	// bit is set TRUE if controls are enabled  
#define M_ZLIMIT	(BYTE)0x02  // bit is set TRUE if lower limit has been set
#define M_DISABLE	(BYTE)0x04  // bit is set TRUE if in manual mode
#define M_HAND		(BYTE)0x08  // bit is set TRUE if hand controller is connected
#define M_FOOT		(BYTE)0x10  // bit is set TRUE if foot controller is connected

/* errors */
//#define NO_ERROR		0
#define LENGTH_ERROR		(BYTE)0xa0
#define COMMAND_CODE_ERROR	(BYTE)0xa1
#define SEND_ERROR		(BYTE)0xa2
#define SEND_LENGTH_ERROR	(BYTE)0xa3
#define RECEIVE_ERROR		(BYTE)0xa4

/* ERROR coming from the TMS */
#define COM_LENGTH_ERROR	(BYTE)0xe0
#define COM_CODE_ERROR		(BYTE)0xe1
#define MEM_IDX_ERROR		(BYTE)0xe2
#define MEM_SAVE_ERROR		(BYTE)0xe3
#define MEM_RETURN_ERROR	(BYTE)0xe4
#define MEM_READ_ERROR		(BYTE)0xe5
#define MOVE_TIP_ERROR		(BYTE)0xe6
#define MOVE_JNT_ERROR		(BYTE)0xe7
#define JNT_IDX_ERROR		(BYTE)0xe8

#define TRAJ_TIP_ERROR		(BYTE)0xea
#define TRAJ_QUEUE_ERROR	(BYTE)0xeb
#define START_ERROR			(BYTE)0xec
#define SRAM_TEST_ERROR		(BYTE)0xed
#define DLOAD_ENTRY_ERROR	(BYTE)0xee
#define DLOAD_ERROR			(BYTE)0xef
#define ILLEGAL_ADDR_ERROR	(BYTE)0xf0
#define ILLEGAL_SIZE_ERROR	(BYTE)0xf1
#define TRAJ_JNT_ERROR		(BYTE)0xf2
#define BRAKE_ERROR			(BYTE)0xf3

/* commands */
#define BOUNCE_FLOATS	(BYTE)0x20
#define DOWNLOAD	(BYTE)0x21
#define EXECUTE		(BYTE)0x22
#define UART_REGS	(BYTE)0x23
#define SRAM_TEST	(BYTE)0x24
#define DUMP_MEM	(BYTE)0x25
#define DLOAD_DATA	(BYTE)0x26
#define DLOAD_END	(BYTE)0x27
#define CANCEL_COMMAND	(BYTE)0x28
#define JT_POS_REQ	(BYTE)0x80
#define CART_TIP_POS	(BYTE)0x81
#define CONT_JOINT_POS	(BYTE)0x82
#define HAND_STATUS	(BYTE)0x83
#define FOOT_STATUS	(BYTE)0x84
#define SERVO		(BYTE)0x85

#define BRAKE		(BYTE)0x86
#define BRAKE_COM	(BYTE)0x86

#define PLAY_VFB	(BYTE)0x87

#define SET_ZLIMIT	(BYTE)0x89
#define AESOP_STATUS	(BYTE)0x8a
#define HOME		(BYTE)0x40
#define MOVE_UP		(BYTE)0x41
#define MOVE_DOWN	(BYTE)0x42
#define MOVE_RIGHT	(BYTE)0x43
#define MOVE_LEFT	(BYTE)0x44
#define ZOOM_IN		(BYTE)0x45
#define ZOOM_OUT	(BYTE)0x46
#define STOP		(BYTE)0x47
#define START		(BYTE)0x48
#define CLEAR		(BYTE)0x49
#define MOVE_REL	(BYTE)0x50
#define MOVE_ABS	(BYTE)0x51
#define MOVE_TIP_REL	(BYTE)0x52
#define MOVE_TIP_ABS	(BYTE)0x53
#define MEM_READ	(BYTE)0x54
#define MEM_SAVE	(BYTE)0x55
#define MEM_RETURN	(BYTE)0x56
#define JNT_TRAJ_LSPB	(BYTE)0x57
#define JNT_TRAJ_CS		(BYTE)0x58
#define TIP_TRAJ_STEP	(BYTE)0x59
#define TIP_TRAJ_LSPB	(BYTE)0x60
#define TIP_TRAJ_CS		(BYTE)0x61
//#define MOVE_JOINT_VEL	(BYTE) 0x62
#define MOVE_ALLJOINTS_VEL (BYTE) 0x62

#define NUMPOS 7 //number of floats required from AESOP for position

typedef unsigned int		  UINT;
typedef unsigned long		  ULONG;



#define HSTAT_NO_CMD 0x0		// No commmand fromn voice, hand, foot etc. buttons 
                                    
// Voice status commands            
// The following bytes are sent from the controller every time
// a voice command is recognized and implemented at the controller CPU
#define HSTAT_VCI_SET_ZLIMIT	0x01		// "Set Lowerlimit"
#define HSTAT_VCI_MOVE_UP		0x02    	// "Move Up"
#define HSTAT_VCI_MOVE_DOWN		0x03    	// "Move Down"
#define HSTAT_VCI_MOVE_RIGHT	0x04    	// "Move Right"
#define HSTAT_VCI_MOVE_LEFT		0x05    	// "Move Left"
#define HSTAT_VCI_ZOOM_IN		0x06    	// "Move In"
#define HSTAT_VCI_ZOOM_OUT		0x07   		// "Move Back"
#define HSTAT_VCI_STOPROBOT		0x08    	// "Stop"
#define HSTAT_VCI_MEM_SAVE		0x09   		// "Save _" (argument is value 1 or 2)
#define HSTAT_VCI_MEM_RETURN	0x0a    	// "Return _" (argument is value 1 or 2)
#define HSTAT_VCI_INCR_UP		0x0b    	// "Up"
#define HSTAT_VCI_INCR_DOWN		0x0c    	// "Down"
#define HSTAT_VCI_INCR_RIGHT	0x0d    	// "Right"
#define HSTAT_VCI_INCR_LEFT		0x0e    	// "Left"
#define HSTAT_VCI_INCR_IN		0x0f   		// "In"
#define HSTAT_VCI_INCR_OUT		0x10   		// "Back"
#define HSTAT_VCI_PAUSEROBOT	0x11    	// <VCI card tells controller to stop e.g because of noise>
#define HSTAT_VCI_COMMAND_MODE  0x12		// "Aesop"

// Hand Status commands
// The following bytes are sent when the hand controller is used
#define HSTAT_HBUT_MEM_SAVE		0x20   		// Save position _	(argument is value 1, 2, 3 or 4)
#define HSTAT_HBUT_MEM_RETURN	0x21		// Return position _ (argument is value 1, 2, 3 or 4)
#define HSTAT_HBUT_SET_ZLIMIT	0x22		// Set LL
#define HSTAT_HBUT_CLEAR_ZLIMIT	0x23    	// Clear LL
#define HSTAT_HBUT_MOVE_IN		0x24    	// Move In
#define HSTAT_HBUT_MOVE_OUT		0x25   		// Move Back
#define HSTAT_HBUT_DISK			0x26        // Disk touched (UP-DOWN LEFT RIGHT)
                
// Foot Status commands    
// The following bytes are sent when the foot controller is used
#define HSTAT_FBUT_MEM_SAVE		0x30   		// Save position _  (argument is value 1 or 2)
#define HSTAT_FBUT_MEM_RETURN	0x31    	// Return position _ (argument is value 1 or 2)
#define HSTAT_FBUT_MOVE_IN		0x33   		// Move In
#define HSTAT_FBUT_MOVE_OUT		0x34    	// Move Back
#define HSTAT_FBUT_DISK			0x35		// Disk touched (UP-DOWN LEFT RIGHT)

// LL Button Status Command          
// The following bytes are sent when the LL button on the arm is pressed
#define HSTAT_LLBUT_SET_ZLIMIT 		0x40	// Set LL
#define HSTAT_LLBUT_CLEAR_ZLIMIT 	0x41   	// Clear LL

// 3D Button Status Command
// The following bytes are sent when the 3D button on the arm is pressed
#define HSTAT_3DB_MANMOD		0x50   	// 3d button is pressed -> manual mode
#define HSTAT_3DB_MOVE_UP		0x51	// Move Up
#define HSTAT_3DB_MOVE_DOWN		0x52    // Move Down











// AESOP Communications Interface
BOOL Initialize(long port);
int Send(BYTE *pbuf, int num);
int Receive(BYTE *pbuf);

// AESOP Commands
void PacFloat(BYTE *OutBuf, int sub, float number);
void UnpacFloat(float *number, BYTE *InBuf, int sub);
UINT C4x_To_IEEE_32(void *vp_memval, UINT u_length);
UINT IEEE_To_C4x_32(void *vp_memdata, UINT u_length);
int BounceFloats(float inf[6], float outf[6]);
int JtPosReq(float position[NUMPOS]);
int CartTipPos(float position[NUMPOS]);
int ContJointPos(float period);
int ProcessJntPos(BYTE *buf, int len, float *time, float jp[6]);
int HandStatus(int *connected);
int FootStatus(int *connected);
int Servo(int joint, BYTE command[]);
int Brake(BYTE command[]);
int PlayVFBMessage(BYTE type);
int SetZLimit(BYTE reset);
int AesopStatus(BYTE *robotStat, BYTE *last_command, BYTE *last_cmdarg);
int Home();
int MoveUp(BYTE speed);
int MoveDown(BYTE speed);
int MoveRight(BYTE speed);
int MoveLeft(BYTE speed);
int ZoomIn(BYTE speed);
int ZoomOut(BYTE speed);
int Stop();
int Start();
int Clear();
int MoveJntRel(BYTE joint, float delta);
int MoveJntAbs(BYTE joint, float angle);
int MoveJointContinous(BYTE joint, float speed);
int MoveAllJointsContinous(float *speed);
int MoveTipRel(float vector[3], float mag);
int MoveTipAbs(float position[3]);
int MemRead(BYTE mem, float position[6]);
int MemSave(BYTE mem);
int MemReturn(BYTE mem);
int JntTrajLSPB(BYTE joint, float posF, float vel, float acc);
int JntTrajCS(BYTE joint, float posF, float velF);
int TipTrajStep(float posI[3], float posF[3]);
int TipTrajLSPB(float posF[3], float vel, float acc);
int TipTrajCS(float posF[3]);
int UARTread(BYTE regs[7]);
int SRAMtest(long begin_addr, long end_addr);
int MemDump(long begin_addr, int size, unsigned long *buf);
int Execute(long addr);

int Download(unsigned long *execAddr, int execflag, BYTE *fname);
int send_cmi(FILE* fp, unsigned long *execAddr);
void PrintRobotStat(int rstat, int lc, int lcarg);

#endif 
