//////////////////////////////////////////////////////////////////////
// COMPUTER MOTION INCORPORATION                  COMPANY CONFIDENTIAL
//
// Class:   IOL_HP_CS (Hamid Packet)
// Needs:   hp.hpp
//
// Version  When      Who               Why
// -------  ----      ---               ---
// 1.0      01/01/96  Daryl Fortney     Initial Design
//
// Copyright 1996 Computer Motion Incorporation.  All rights reserved.
//////////////////////////////////////////////////////////////////////
// Method Defintions
//////////////////////////////////////////////////////////////////////
#ifndef HP_CS_CPP
#define HP_CS_CPP

// Dependencies
#include "hp_cs.hpp"

//////////////////////////////////////////////////////////////////////
// Maintenance Methods
//////////////////////////////////////////////////////////////////////
//
// Print() - This method prints out all class attributes.
//
void IOL_HP_CS::Print(){

	cerr<<"IOL_HP_CS:"<<endl
		<<"========================================"<<endl
		<<"TxLength         R  = "<<TXL<<" Bytes"<<endl
		<<"TxChkSum         R  = "<<TXCS<<endl
		<<"RxLength         R  = "<<RXL<<" Bytes"<<endl
		<<"RxChkSum         R  = "<<RXCS<<endl;
	L1.Print();
}


BOOL IOL_HP_CS::Open(char *p,long m) {
	if(!L1.Open(p,m))
		return FALSE;	
	
	L1.Timeout(HP_TIMEOUT); 
	return TRUE;
}


BOOL IOL_HP_CS::Close(void) {
	return L1.Close(); 
}



//////////////////////////////////////////////////////////////////////
// Tx Stream Methods
//////////////////////////////////////////////////////////////////////
//
// Tx() - This method buffers the Tx Stream until it is actually
// transmitted through TxPacket. It also computes the running Checksum.
//
long IOL_HP_CS::Tx(void *block,long num,long until){
	unsigned char *blk=(unsigned char *)block;

	// Buffer Data and Compute Checksum
	long	i;
	if(until>=0)
		for(i=0;i<num && blk[i]!=until && TXL<MAX_NUM;i++){
			TX[TXL++]=blk[i];
			TXCS+=blk[i];
		}
	else
		for(i=0;i<num && TXL<MAX_NUM;i++){
			TX[TXL++]=blk[i];
			TXCS+=blk[i];
		}

	return i;
}

//
// TxPacket() - This method forms a valid Hamid Packet from the
// buffered Tx Stream data and attempts transmission. On success
// it returns the number of bytes actually transmitted and performs
// a TxReset().
//
long IOL_HP_CS::TxPacket(){

	// Guarantee Data Exists
	if (TXL==0) return 0;

	// Form Hamid Packet (ENQ RESERVED Length Data... Checksum EOT)
	long			num=0;
	unsigned char	packet[MAX_NUM*2+10];
	unsigned char	cs;
	
	// Update Checksum
	cs=(unsigned char)((CS_KEY-(TXCS+RESERVED+TXL))&0xff);

	packet[num++]=IOL_ENQ;
	Pack(RESERVED,packet,&num);
	Pack(TXL,packet,&num);
	for(long i=0;i<TXL;i++){
		Pack(TX[i],packet,&num);
		if(i==TXL-1) 
			Pack(cs,packet,&num);
	}
	packet[num++]=IOL_EOT;

	// Attempt Tx Packet While Response is NAK
	unsigned char	respond=0;
	do{
		if(L1.Tx(packet,num)!=num) {
			printf("ERROR: HP_CS TxPacket() could not send packet\n");
			return 0;
		}
//		if(L1.Rx(&respond,1)!=1) { 
//			printf("ERROR: HP_CS TxPacket() did not receive response\n");
//			return 0;
//		}
//		if((respond!=IOL_NAK) && (respond!=IOL_ACK)){
//			printf("WARNING: HP_CS TxPacket() received %lx response\n",respond);
//			return 0;
//		}
	}while(respond==IOL_NAK);

	TxReset();

	return num;
}

//////////////////////////////////////////////////////////////////////
// Rx Stream Methods
//////////////////////////////////////////////////////////////////////
//
// Rx() - This method unbuffers the Rx Stream that has been received by
// RxPacket().
//
long IOL_HP_CS::Rx(void *block,long num,long until){
	unsigned char *blk=(unsigned char *)block;

	// UnBuffer Data
	long	i;
	if(until>=0)
		for(i=0;i<num && RX[i]!=until && RXL;i++)
			blk[i]=RX[--RXL];
	else
		for(i=0;i<num && RXL;i++)
			blk[i]=RX[--RXL];

	return i;
}

//
// RxPacket() - This method performs a RxReset() and then receives a new
// valid Hamid Packet and places the data in the Rx Stream for unbuffering
// by Rx().
//
long IOL_HP_CS::RxPacket(){

	// Guarantee Packet Waiting
	if(L1.Rx()==0) return 0;

	RxReset();

	// Try to Rx Hamid Packet Data Only
	unsigned char	nak=IOL_NAK;
	unsigned char	ack=IOL_ACK;
	long			num=0;
	unsigned char	packet[MAX_NUM+5];
	while(1){

		// Get Next Character
		if(L1.Rx(&packet[num],1)==0) 
			return 0;
		
		// End of Packet?
		if(packet[num]==IOL_EOT) 
			break;

		// Start of Packet?
		if(packet[num]==IOL_ENQ) 
			num=0;
		else{

			// Extended Character Check
			if(packet[num]==IOL_ETB){
				while(packet[num]==IOL_ETB)
					if(L1.Rx(&packet[num],1)==0) 
						return 0;
				packet[num]&=0x1F;
			}else{
				// Filter Out Exception Characters
				if(Exception(packet[num])){
					if(packet[num]!=IOL_ACK)
						printf("WARNING: HP Discarding Embedded %lx\n",(long)packet[num]);
					continue;
				}
			}

			// Maximum Length Check
			if(++num>=MAX_NUM+5){ 
				L1.Tx(&nak); 
				return 0;	
			}
		}
	}
	num--;

	if(num<3) { 
		L1.Tx(&nak); 
		return 0; 
	}

	// Parse Hamid Packet (RESERVED Length Data... Checksum)
	if(packet[1]!=(unsigned char)(num-2)){
		L1.Tx(&nak);
		return 0;
	}

	// Reset & Fill Rx Stream
	unsigned int cs=packet[0]+packet[1];
	for(int i=2;i<num;i++){
		cs+=packet[i];
		RX[(num-3)-RXL++]=packet[i];
	}
	
	// Validate Checksum
	if((unsigned char)((CS_KEY-cs)&0xff)!=packet[num]){
		L1.Tx(&nak);
		return 0;
	}

	// Acknowledge Valid Packet
	L1.Tx(&ack);

	return RXL;
}

void IOL_HP_CS::Pack(unsigned char d,unsigned char* p,long* num){
	if(Exception(d)){
		p[(*num)++]=IOL_ETB;
		p[(*num)++]=(d|0xE0);
	}else
		p[(*num)++]=d;
}

long IOL_HP_CS::Exception(unsigned char v){

	return (v>=IOL_EOT && v<=IOL_ACK) || (v>=IOL_NAK && v<=IOL_CAN);
}

//////////////////////////////////////////////////////////////////////
// Old Protocol Compatibility
//////////////////////////////////////////////////////////////////////

int IOL_HP_CS::SerialInit(int buffers, int retries, long baud, int port){

	// Open Protocol
	if(!Open(UART_NAME[port]))
		return 1;

	return 0;
}

//////////////////////////////////////////////////////////////////////
// SerialSend: Sends the given data on the serial bus.
//
// Returns:	  0 Data Transmit Success
//            1 Data Length Overflow
//            2 Data Transmit Error
//////////////////////////////////////////////////////////////////////
int IOL_HP_CS::SerialSend(char *pbuf, int num){

	// Attempt Packetize Data
	if(Tx(pbuf,num,IOL_ALL)!=num)
		return 1;

	// Attempt Packet Send
	if(!TxPacket())
		return 2;

	return 0;
}

//////////////////////////////////////////////////////////////////////
// SerialReceive: Receives the next available packet from the serial 
// port.
//
// Returns: 0 if no data is available
//			otherwise returns the length of the packet found
//
//////////////////////////////////////////////////////////////////////
int IOL_HP_CS::SerialReceive(char *pbuf){
	long	rx;

	// Attempt Receive Packet
	rx=RxPacket();
	if(rx==0)
		return 0;

	// Attempt Unpacketize Data
	return Rx(pbuf,rx,IOL_ALL);
}

#endif